// Config file
new const cfgfile[]  = 	"RateChecker.cfg"

// Maximum cvar items 256 items should be enough
#define MAXITEMS 	256

/*************************/
/* Code starts from here */
/*************************/

#include <amxmodx>
#include <amxmisc>

#define VERSION "Rev.2"
#define PLUGIN "RateChecker"
#define AUTHOR "TjaardA-Powered"

new client_isettings[33] // Invalid settings count
new g_configfile[128]

// pcvars
new cvar_ratecheck, cvar_messagetype, 
    cvar_chkdelay, cvar_minupdaterate, 
    cvar_mincmdrate, cvar_minrate, 
    cvar_maxcmdbackup, cvar_maxinvalid, 
    cvar_showinfo
   
// Initiate Plugin
public plugin_init() 
{
	register_plugin(PLUGIN, VERSION, AUTHOR)
	cvar_ratecheck = register_cvar("cc_ratecheck", "1")
	cvar_messagetype = register_cvar("cc_messagetype", "2")
	cvar_chkdelay = register_cvar("cc_chkdelay", "36")
	cvar_minupdaterate = register_cvar("cc_minupdaterate", "100")
	cvar_mincmdrate = register_cvar("cc_mincmdrate", "100")
	cvar_minrate = register_cvar("cc_minrate", "20000")
	cvar_maxcmdbackup = register_cvar("cc_maxcmdbackup", "5")
	cvar_maxinvalid = register_cvar("cc_maxinvalid", "10")
	cvar_showinfo = register_cvar("cc_showinfo", "1")
}
// Plugin Configure
public plugin_cfg()
{
	new cfgdir[32]
	// Load CFG file
	get_configsdir(cfgdir, 31)
	formatex(g_configfile, 63, "%s/%s", cfgdir, cfgfile)
	if (file_exists(g_configfile)) 
	{
		server_print("[RateChecker] Loading %s", cfgfile)
		server_cmd("exec %s", g_configfile)
	}
	return PLUGIN_CONTINUE
}
// Client left the server 
public client_disconnect(id) 
{
	{
		new name[32]
		get_user_name(id, name, 31)
		client_print(0, print_chat, "%s, disconnected.", name)
	}
	// Remove checker task
	if (task_exists(id))
		remove_task(id)
		
	return PLUGIN_CONTINUE
}

// Client put in server
public client_putinserver(id) 
{
	new is_bot = is_user_bot(id)
	new is_hltv = is_user_hltv(id)
	if (!is_bot && !is_hltv) 
	{
		new param[1]
		param[0] = id
		set_task(5.0, "startchk", id, param, 1)
	}
	return PLUGIN_CONTINUE
}
// Start Checks
public startchk(param[]) 
{
	new id = param[0]
	// Print crap in console
	if (get_pcvar_num(cvar_showinfo))
	{
		client_print(id, print_console, "--------------[ Rate Checker ]-------------")
		client_print(id, print_console, "Version: %s", VERSION)
		client_print(id, print_console, "Rate check: %s", get_num_str(cvar_ratecheck))
		client_print(id, print_console, "----------------------------------------------")
		client_print(id, print_console, "Min rate: %d", get_pcvar_num(cvar_minrate))
		client_print(id, print_console, "Min updaterate: %d", get_pcvar_num(cvar_minupdaterate))
		client_print(id, print_console, "Min cmdrate: %d", get_pcvar_num(cvar_mincmdrate))
		client_print(id, print_console, "Max cmdbackup: %d", get_pcvar_num(cvar_maxcmdbackup))
		client_print(id, print_console, "-----------------------------------------------")
	}
	
	// Run all checks before check cycle starts.
	set_task(float(get_pcvar_num(cvar_chkdelay)), "CheckClient", id, param, 1, "b")
	return PLUGIN_HANDLED
}
get_num_str(pcvar) {
	new tmp[9]
	switch(get_pcvar_num(pcvar))
	{
		case 0:
			tmp = "disabled"
		case 1:
			tmp = "enabled"
	}
	return tmp
}
public CheckClient(param[]) 
{
	new id = param[0]
	
	if (!is_user_connected(id)) 
	{
		remove_task(id)
		return PLUGIN_HANDLED
	}
		
	// Check rates
	if (get_pcvar_num(cvar_ratecheck)) 
	{
		query_client_cvar(id, "rate", "ClientCvarResult")
		query_client_cvar(id, "cl_updaterate", "ClientCvarResult")
		query_client_cvar(id, "cl_cmdrate", "ClientCvarResult")
		query_client_cvar(id, "cl_cmdbackup", "ClientCvarResult")
	}
	return PLUGIN_CONTINUE
}
public ClientCvarResult(id, const cvar[], const value[]) 
{
	new minrate, mincmdrate, minupdrate, maxcmdbackup
	minrate = get_pcvar_num(cvar_minrate)
	mincmdrate = get_pcvar_num(cvar_mincmdrate)
	minupdrate = get_pcvar_num(cvar_minupdaterate)
	maxcmdbackup = get_pcvar_num(cvar_maxcmdbackup)
	new Float:f_value = str_to_float(value)
	if (equali(value, "Bad CVAR request"))
		return PLUGIN_HANDLED
	// Check Rates
	else if ((equal(cvar, "rate")) && (f_value < minrate)) 
	{  
		cc_messagetype(id, cvar,  value) 
		console_cmd(id, "rate %d", minrate)
		return PLUGIN_HANDLED
	}
	else if ((equal(cvar, "cl_updaterate")) && (f_value < minupdrate)) 
	{ 
		cc_messagetype(id, cvar, value)
		console_cmd(id, "cl_updaterate %d", minupdrate)
		return PLUGIN_HANDLED
	}
	else if ((equal(cvar, "cl_cmdrate")) && (f_value < mincmdrate)) 
	{ 
		cc_messagetype(id, cvar,value)
		console_cmd(id, "cl_cmdrate %d", mincmdrate)
		return PLUGIN_HANDLED
	}
	else if ((equal(cvar, "cl_cmdbackup")) && (f_value > maxcmdbackup)) 
	{ 
		cc_messagetype(id, cvar,value)
		console_cmd(id, "cl_cmdbackup %d", maxcmdbackup)
		return PLUGIN_HANDLED
	}
	return PLUGIN_CONTINUE
}
cc_messagetype(id, const cvar[], const value[]) 
{
	new name[32], is_admin, is_hltv, num
	get_user_name(id, name, 31)
	new type = get_pcvar_num(cvar_messagetype)
	/*
		type 1: Admins only
		type 2: Checked Player + Admins
		type 3: All Players
	*/
	if (type == 3)
		client_print(0, print_chat, "%d %s: %s %s", id, name, cvar, value)
	else
	{
		new players[32], player
		get_players(players, num, "c")
		for (new i = 0;i < num;i++) 
		{
			player = players[i]
			is_hltv = is_user_hltv(player)
			if (!is_hltv)
			{
				is_admin = is_user_admin(player)
				switch(type)
				{
					case 1:
					{
						if (is_admin) 
							client_print(player, print_chat, "%d %s: %s %s", id, name, cvar, value)
					}
					case 2:
					{
						client_print(id, print_chat, "%d %s: %s %s", id, name ,cvar, value)
						if (is_admin && id != player) 
							client_print(player, print_chat, "%d %s: %s %s", id, name, cvar, value)
					}
				}
			}
		}
	}
	// Invalid settings counter
	num = get_pcvar_num(cvar_maxinvalid) 
	if (num > 0)
	{
		client_isettings[id]++
		if (client_isettings[id] >= num)
		{
			new uID = get_user_userid(id)
			server_cmd("banid 1 #%d", uID)
			client_print(id, print_console, "kicked due to constant changing to invalid settings!!")
			server_cmd("kick #%d ^"kicked due to constant changing to invalid settings!^"", uID)
			client_print(0, print_chat, "[RateChecker] %s, got kicked due to constant changing to invalid settings!", name)
		}
	}
	return PLUGIN_CONTINUE
}